@@ -130,6 +130,7 @@ class PureScheduler:                                    # pylint: disable=r0902
         # avoid multiple shutdowns
         self._did_shutdown = False
 
+
     # think of an scheduler as a set of jobs
     def update(self, jobs):
         """
@@ -186,6 +187,13 @@ class PureScheduler:                                    # pylint: disable=r0902
         """
         return len(self.jobs)
 
+    def __iter__(self):
+        """
+        iterating on a scheduler amounts to iterating on its jobs
+        """
+        return iter(self.jobs)
+
+
     def failed_time_out(self):
         """
         Returns:
@@ -396,6 +404,47 @@ class PureScheduler:                                    # pylint: disable=r0902
             if not job._s_successors:                   # pylint: disable=w0212
                 yield job
 
+
+    def downstream_jobs(self, job):
+        """
+        return an iterator on the jobs in s that require on job
+        in no particular order
+        """
+        return (downstream for downstream in self if job in downstream.required)
+
+
+    def bypass_and_remove(self, job: AbstractJob):
+        """
+        job is assumed to be part of the scheduler (and `ValueError` is raised otherwise)
+
+        this method will remove `job` from the scheduler, while preserving the logic;
+        for achieving this, all the requirement links will be redirected to bypass that job
+
+        more formally, for each job `upstream` that `job` requires, the algorithm will create
+        requirement links from all `downstream` jobs that require `job`
+
+        for now, nesting is not supported, that is to say, the job needs to be an actual
+        member of this scheduler, and not of a nested scheduler within;
+        jobs in a `Sequence` work fine though
+
+        """
+        if job not in self.jobs:
+            raise ValueError(f"job {job} is not in {self}")
+
+        upstreams = job.required
+        downstreams = {down for down in self.jobs if job in down.required}
+
+        # add new requires between downstreams and upstreams
+        for up in upstreams:
+            for down in downstreams:
+                down.requires(up)
+        # remove job from the downstreams requirements
+        for down in downstreams:
+            down.required.remove(job)
+        # remove the job altogether
+        self.jobs.remove(job)
+
+
     def _entry_csv(self):
         result = ", ".join(job.repr_id()
                            for job in self.entry_jobs())
@@ -748,8 +797,12 @@ class PureScheduler:                                    # pylint: disable=r0902
         Also, the canonical name for this is ``run()`` but for historical
         reasons you can also use ``orchestrate()`` as an alias for ``run()``.
         """
-        return asyncio.get_event_loop().run_until_complete(
-            self.co_run(*args, **kwds))
+        try:
+            loop = asyncio.get_event_loop()
+        except RuntimeError:
+            loop = asyncio.new_event_loop()
+            asyncio.set_event_loop(loop)
+        return loop.run_until_complete(self.co_run(*args, **kwds))
 
     # define the alias for legacy
     orchestrate = run
@@ -1209,6 +1262,7 @@ DOT_%28graph_description_language%29
         result += "}\n"
         return result
 
+
     def export_as_dotfile(self, filename):
         """
         This method does not require ``graphviz`` to be installed, it
@@ -1235,6 +1289,7 @@ DOT_%28graph_description_language%29
             output.write(self.dot_format())
         return "(Over)wrote {}".format(filename)
 
+
     def graph(self):
         """
         Returns:
@@ -1263,13 +1318,19 @@ DOT_%28graph_description_language%29
         from graphviz import Source
         return Source(source=self.dot_format())
 
-    def export_as_pngfile(self, filename):
+
+    def export_as_graphic(self, filename, suffix):
         """
-        Convenience wrapper that creates a png file. Like :meth:`graph()`,
+        Convenience wrapper that creates a graphic output file.
+        Like :meth:`graph()`,
         it requires the ``graphviz`` package to be installed.
+        See also :meth:`export_as_pngfile()`, which is a shortcut to
+        using this method with `suffix="png"`
 
         Parameters:
-          filename: output filename, without the ``.png`` extension
+          filename: output filename, without the extension
+          suffix: one of the suffixes supported by `dot -T`, e.g. `png` or `svg`
+
         Returns:
           created file name
 
@@ -1283,9 +1344,31 @@ DOT_%28graph_description_language%29
         # of the .dot file; that is dangerous when using e.g.
         #    scheduler.export_as_pngfile(__file__)
         import os
-        dotfile = "{}.dot".format(filename)
-        pngfile = "{}.png".format(filename)
+        dotfile = f"{filename}.dot"
         self.export_as_dotfile(dotfile)
-        os.system("dot -Tpng {} -o {}"
-                  .format(dotfile, pngfile))
+        pngfile = f"{filename}.{suffix}"
+        os.system(f"dot -T{suffix} {dotfile} -o {pngfile}")
         return pngfile
+
+
+    def export_as_pngfile(self, filename):
+        """
+        Shortcut to :meth:`export_as_graphic()` with `suffix="png"`
+
+        Parameters:
+          filename: output filename, without the ``.png`` extension
+        Returns:
+          created file name
+        """
+        return self.export_as_graphic(filename, "png")
+
+    def export_as_svgfile(self, filename):
+        """
+        Shortcut to :meth:`export_as_graphic()` with `suffix="svg"`
+
+        Parameters:
+          filename: output filename, without the ``.svg`` extension
+        Returns:
+          created file name
+        """
+        return self.export_as_graphic(filename, "svg")
